@@ -1,13 +1,13 @@
-# Copyrights 2013-2014 by [Mark Overmeer].
+# Copyrights 2013-2015 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.01.
+# Pod stripped from pm file by OODoc 2.02.
 use warnings;
 use strict;
 
 package Net::Domain::TMCH;
 use vars '$VERSION';
-$VERSION = '0.17';
+$VERSION = '0.18';
 
 use base 'Exporter';
 
@@ -18,7 +18,7 @@ use Net::Domain::TMCH::CRL     ();
 use Net::Domain::SMD::RL       ();
 
 use Crypt::OpenSSL::VerifyX509 ();
-use Crypt::OpenSSL::X509   ();
+use Crypt::OpenSSL::X509   qw(FORMAT_ASN1 FORMAT_PEM);
 use File::Basename         qw(dirname);
 use File::Spec::Functions  qw(catfile);
 use Scalar::Util           qw(blessed);
@@ -29,31 +29,43 @@ use constant
   , TMV_CRL_PILOT => 'http://crl.icann.org/tmch_pilot.crl'
   };
 
+sub icannCert($) { catfile dirname(__FILE__), 'TMCH', 'icann', "$_[1].pem" }
+
 
 sub new($%) { my ($class, %args) = @_; (bless {}, $class)->init(\%args) }
 
 sub init($)
 {   my ($self, $args) = @_;
+
+    my $tmv = $args->{tmv_certificate};
+    if($tmv && !(blessed $tmv && $tmv->isa('Crypt::OpenSSL::X509')))
+    {   my $read = eval { Crypt::OpenSSL::X509->new_from_file($tmv) };
+        $@ and error __x"cannot read certificate from {file}: {err}"
+          , file => $tmv, err => $@;
+        $tmv = $read;
+    }
+
     $self->{NDT_smds} = $args->{smds_admin} ||
         Net::Domain::SMD::Schema->new
-         ( prepare       => 'READER'
-         , auto_datetime => $args->{auto_datetime}
-         );
+          ( auto_datetime   => $args->{auto_datetime}
+          , tmv_certificate => $tmv
+          );
 
     my $pilot    = $self->{NDT_pilot} = $args->{is_pilot};
     my $stage    = $pilot ? 'tmch_pilot' : 'tmch';
+    my $tmch_pem = $args->{tmch_certificate} || $self->icannCert($stage);
 
-    my $tmch_pem = $args->{tmch_certificate}
-     || catfile dirname(__FILE__), 'TMCH', 'icann', "$stage.pem";
+    # user will not understand the errors from module ::X509
+    use filetest 'access';
+    -r $tmch_pem
+        or error __x"cannot read PEM from {fn}", fn => $tmch_pem;
 
     $self->{NDT_tmch_cert} = Crypt::OpenSSL::X509->new_from_file($tmch_pem);
     $self->{NDT_tmch_ca}   = Crypt::OpenSSL::VerifyX509->new($tmch_pem);
-
-    $self->{NDT_crl}   = $self->_crl($args->{cert_revocations}
+    $self->{NDT_smdrl}     = [ $self->_smdrl($args->{smd_revocations}) ];
+    $self->{NDT_crl}       = $self->_crl($args->{cert_revocations}
        || ($pilot ? TMV_CRL_PILOT : TMV_CRL_LIVE));
 
-    $self->{NDT_smdrl} = [ $self->_smdrl($args->{smd_revocations}) ];
-
     $self;
 }
 
@@ -148,4 +160,10 @@ sub smd($%)
     $smd;
 }
 
+
+sub createSignedMark($%)
+{   my ($self, $doc, $data, %args) = @_;
+    $self->smdAdmin->createSignedMark($doc, $data, \%args);
+}
+
 1;
